# To test your own install, just copy this file to t/config.pl and make
# appropriate changes to the values.

use strict;
use warnings;

{
    partner  => 'PayPal',
    password => 'seekrit',
    user     => 'someuser',
    vendor   => 'somevendor',
};
