package WebService::PayPal::PaymentsAdvanced::Role::HasParams;
$WebService::PayPal::PaymentsAdvanced::Role::HasParams::VERSION = '0.000005';
use Moo::Role;

use Types::Standard qw( HashRef );

has params => (
    is       => 'ro',
    isa      => HashRef,
    required => 1,
);

1;

=pod

=encoding UTF-8

=head1 NAME

WebService::PayPal::PaymentsAdvanced::Role::HasParams - Role which provides a params attribute to exception classes.

=head1 VERSION

version 0.000005

=head1 AUTHOR

Olaf Alders <olaf@wundercounter.com>

=head1 COPYRIGHT AND LICENSE

This software is Copyright (c) 2015 by MaxMind, Inc..

This is free software, licensed under:

  The Artistic License 2.0 (GPL Compatible)

=cut

__END__
#ABSTRACT: Role which provides a params attribute to exception classes.
